@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2010 - 2016,  Ron <ron@debian.org>
+//  Copyright 2010 - 2018,  Ron <ron@debian.org>
 
 #ifndef _BB_FTDI_DEVICE_H
 #define _BB_FTDI_DEVICE_H
@@ -331,7 +331,7 @@ namespace BitB
             if( ret != 2 )
                 ThrowError( _("FTDI: get modem status returned %d bytes"), ret );
 
-            return (ms[0] << 8) | ms[1];
+            return uint16_t( (ms[0] << 8) | ms[1] );
 
         } //}}}
 
@@ -370,7 +370,7 @@ namespace BitB
                 pthread_setcancelstate( PTHREAD_CANCEL_DISABLE, &oldstate );
 
                 int xfer;
-                int n   = std::min( len, m_chunksize );
+                int n   = int(std::min( len, m_chunksize ));
                 int ret = libusb_bulk_transfer( *m_dh, m_epout, b, n, &xfer, m_timeout );
 
                 pthread_setcancelstate( oldstate, NULL );
@@ -384,7 +384,7 @@ namespace BitB
                         if( __builtin_expect(xfer < 0 || size_t(xfer) > len,0) )
                             ThrowError( _("FTDI: OOPS write of %d returned %d ..."), n, xfer );
 
-                        len -= xfer;
+                        len -= unsigned(xfer);
                         b   += xfer;
                         break;
 
@@ -409,13 +409,13 @@ namespace BitB
 
             int     oldstate;
             int     xfer;
-            int     n = std::min( len, m_chunksize );
+            int     n = int(std::min( len, m_chunksize ));
 
             // Ensure we always request a multiple of m_maxpacket, otherwise
             // we can get an overflow from the last packet that is received,
             // since the transfer size isn't sent to the device and it might
             // still send a 'full' packet even if we wanted less than that.
-            n = round_to_maxpacket(n);
+            n = int(round_to_maxpacket(size_t(n)));
 
             pthread_testcancel();
             pthread_setcancelstate( PTHREAD_CANCEL_DISABLE, &oldstate );
@@ -436,7 +436,7 @@ namespace BitB
                     if( __builtin_expect(xfer < 0 || xfer > n, 0) )
                         ThrowError( _("FTDI: OOPS read of %d returned %d ..."), n, xfer );
 
-                    return xfer;
+                    return size_t(xfer);
             }
 
             ThrowUSBError( ret, _("FTDI: read chunk of %d bytes failed"), n );
@@ -824,7 +824,7 @@ namespace BitB
 
             purge_read();
             ftdi_set_special_chars();
-            ftdi_set_latency_timer( m_latency );
+            ftdi_set_latency_timer( uint8_t(m_latency) );
             ftdi_set_flow_control( FLOW_RTS_CTS );
             ftdi_set_bitmode( BITMODE_RESET );
             ftdi_set_bitmode( BITMODE_MPSSE );
