@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2003 - 2017,  Ron <ron@debian.org>
+//  Copyright 2003 - 2018,  Ron <ron@debian.org>
 //
 // This file provides the implementation detail for bit-babbler/log.h
 // which must be defined only once in an application.
@@ -242,7 +242,7 @@ namespace BitB
 
     } //}}}
 
-    std::string timeprintf( std::string format, const timeval &tv )
+    std::string timeprintf( const char *format, const timeval &tv )
     { //{{{
 
         using std::string;
@@ -252,53 +252,48 @@ namespace BitB
         struct tm           tm;
         string::size_type   n = 0;
 
-        // First substitute microseconds if required.
-        while( ( n = format.find( "%%", n, 2 ) ) != string::npos )
-        {
-            string::size_type   end = format.find( 'u', n );
-
-            if( end != string::npos )
-            {
-                string::size_type   len = end - n;
-
-                char    usec[ 7 ];
-                snprintf( usec, sizeof( usec ), "%06ld", long(tv.tv_usec) );
-
-                if( len == 2 )
-                {
-                    format.replace( n, len + 1, usec, 6 );
-                }
-                else if( len == 3 )
-                {
-                    unsigned int width;
-                    if( sscanf( string( format, n + 2, len - 2 ).c_str(), "%u", &width ) == 1 )
-                        format.replace( n, len + 1, usec, std::min(width,6u) );
-                }
-                n = end;
-            }
-        }
-
         // time_t may not be the same size as tv_sec, and indeed on 64-bit Windows
         // (and possibly OpenBSD among others) time_t is long long while tv_sec is
         // only long.
-        time_t  sec = tv.tv_sec;
+        time_t              sec = tv.tv_sec;
 
+        // First format a string according to the standard specifiers.
        #if HAVE_LOCALTIME_R
-        // Then format the rest according to standard specifiers.
         localtime_r( &sec, &tm );
        #else
         memcpy( &tm, localtime( &sec ), sizeof( tm ) );
        #endif
 
+
        #if EM_PLATFORM_MSW
 
+        // We lose the ability to have compile time format checking of strftime
+        // by doing this but for that function it's of fairly limited value and
+        // as of gcc-7 we need to disable it here anyway because it doesn't see
+        // that this function itself is format-checked and so still whines that
+        // the format is not a literal even though it just checked that it is!
+        string  fmt = format;
+
         // Windows doesn't support the %T format option, so expand it manually here.
+        while( ( n = fmt.find( "%T", n, 2 ) ) != string::npos )
+            fmt.replace( n, 2, "%H:%M:%S" );
+
+        n = 0;
+
+        // Windows doesn't support the %F format option, so expand it manually here.
+        while( ( n = fmt.find( "%F", n, 2 ) ) != string::npos )
+            fmt.replace( n, 2, "%Y-%m-%d" );
+
         n = 0;
-        while( ( n = format.find( "%T", n, 2 ) ) != string::npos )
-            format.replace( n, 2, "%H:%M:%S" );
 
        #endif
 
+
+        // gcc-7 complains about the format parameter passed to strftime, even
+        // though it already checked it when this function itself was called.
+      #pragma GCC diagnostic push
+      #pragma GCC diagnostic ignored "-Wformat-nonliteral"
+
        #if EM_PLATFORM_LINUX
 
         // Querying the string length with a NULL for the output is a GNU extension.
@@ -307,7 +302,7 @@ namespace BitB
         // It's also tricky to actually test for that, since on systems where it
         // doesn't work, there is no clear indication except perhaps a segfault.
         // So we'll be conservative and only use it where we know it really works.
-        size_t  size = strftime( NULL, MAX_LEN, format.c_str(), &tm );
+        size_t  size = strftime( NULL, MAX_LEN, format, &tm );
 
        #else
 
@@ -319,8 +314,46 @@ namespace BitB
         {
             char    buf[ size + 1 ];
 
-            if( strftime( buf, size + 1, format.c_str(), &tm ) > 0 )
+           #if EM_PLATFORM_MSW
+
+            if( strftime( buf, size + 1, fmt.c_str(), &tm ) > 0 )
                 timestr = buf;
+
+           #else
+
+            if( strftime( buf, size + 1, format, &tm ) > 0 )
+                timestr = buf;
+
+           #endif
+        }
+
+      #pragma GCC diagnostic pop
+
+
+        // Then substitute microseconds if required.
+        while( ( n = timestr.find( '%', n ) ) != string::npos )
+        {
+            string::size_type   end = timestr.find( 'u', n );
+
+            if( end != string::npos )
+            {
+                string::size_type   len = end - n;
+
+                char    usec[ 7 ];
+                snprintf( usec, sizeof( usec ), "%06ld", long(tv.tv_usec) );
+
+                if( len == 1 )
+                {
+                    timestr.replace( n, len + 1, usec, 6 );
+                }
+                else if( len == 2 )
+                {
+                    unsigned int width;
+                    if( sscanf( string( timestr, n + 1, len - 1 ).c_str(), "%u", &width ) == 1 )
+                        timestr.replace( n, len + 1, usec, std::min(width,6u) );
+                }
+                n = end;
+            }
         }
 
         return timestr;
@@ -362,6 +395,9 @@ namespace BitB
 
    #ifdef _REENTRANT
 
+    #if ! THREAD_STACK_SIZE
+    BB_CONST
+    #endif
     const pthread_attr_t *GetDefaultThreadAttr()
     { //{{{
 
@@ -407,12 +443,17 @@ namespace BitB
 
 #if EM_PLATFORM_MSW
 
-void openlog(const char *ident, int option, int facility)
+BB_CONST void openlog(const char *ident, int option, int facility)
 {
     (void)ident; (void)option; (void)facility;
 }
 
-void vsyslog(int priority, const char *format, va_list ap)
+BB_CONST void syslog(int priority, const char *format, ...)
+{
+    (void)priority; (void)format;
+}
+
+BB_CONST void vsyslog(int priority, const char *format, va_list ap)
 {
     (void)priority; (void)format; (void)ap;
 }
