@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2014 - 2016,  Ron <ron@debian.org>
+//  Copyright 2014 - 2018,  Ron <ron@debian.org>
 
 #ifndef _BB_SECRET_SINK_H
 #define _BB_SECRET_SINK_H
@@ -30,6 +30,16 @@ namespace BitB
             size_t          block_size;
             size_t          bytes;
 
+            Options()
+                : block_delay( 0 )
+                , block_size( 65536 )
+                , bytes( 0 )
+            {}
+
+            // We don't need these anymore with the new seedd config parser.
+            // But hang on to them a bit longer in case something else does.
+           #if 0
+
             Options( const std::string &path, size_t bd = 0, size_t bs = 65536, size_t n = 0 )
                 : devpath( path )
                 , block_delay( bd )
@@ -75,6 +85,8 @@ namespace BitB
 
             } //}}}
 
+           #endif
+
         }; //}}}
 
 
@@ -89,6 +101,8 @@ namespace BitB
         void do_read_thread()
         { //{{{
 
+            SetThreadName( stringprintf("QA %s", m_options.devpath.c_str()).c_str() );
+
             Log<3>( "SecretSink( %s ): begin read_thread\n", m_options.devpath.c_str() );
 
             uint8_t     buf[ m_options.block_size ];
@@ -99,7 +113,7 @@ namespace BitB
             {
                 while( n < m_options.block_size )
                 {
-                    int r = read( m_fd, buf + n, m_options.block_size - n );
+                    ssize_t r = read( m_fd, buf + n, m_options.block_size - n );
 
                     if( r < 0 )
                         throw SystemError( _("SecretSink( %s )::read( %zu ) failed"),
@@ -108,7 +122,7 @@ namespace BitB
                     if( r == 0 )
                         throw Error( _("SecretSink( %s )::read EOF"), m_options.devpath.c_str() );
 
-                    n += r;
+                    n += size_t(r);
                 }
 
                 m_qa.Check( buf, n );
@@ -124,7 +138,7 @@ namespace BitB
                 }
 
                 if( m_options.block_delay )
-                    usleep( m_options.block_delay * 1000 );
+                    usleep( useconds_t(m_options.block_delay * 1000) );
             }
 
         } //}}}
