@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2010 - 2017,  Ron <ron@debian.org>
+//  Copyright 2010 - 2018,  Ron <ron@debian.org>
 
 #ifndef _BB_SECRET_SOURCE_H
 #define _BB_SECRET_SOURCE_H
@@ -183,7 +183,7 @@ namespace BitB
                 if( n != 0 )
                 {
                     try {
-                        sleep_init = StrToScaledUL( arg.substr(0, n) );
+                        sleep_init = StrToScaledU( arg.substr(0, n) );
                     }
                     catch( const std::exception &e )
                     {
@@ -195,7 +195,7 @@ namespace BitB
                 if( n + 1 < arg.size() )
                 {
                     try {
-                        sleep_max = StrToScaledUL( arg.substr(n + 1) );
+                        sleep_max = StrToScaledU( arg.substr(n + 1) );
                     }
                     catch( const std::exception &e )
                     {
@@ -278,11 +278,11 @@ namespace BitB
         { //{{{
 
             if( options.bitrate == m_bitrate )
-                LogMsg<2>( "+ BitBabbler( bitrate %u, fold %u, mask 0x%02x )",
-                                        m_bitrate, m_fold, m_enable_mask );
+                LogMsg<2>( "+ BitBabbler( bitrate %u, fold %u, mask 0x%02x [%02x] )",
+                                m_bitrate, m_fold, options.enable_mask, m_enable_mask );
             else
-                LogMsg<2>( "+ BitBabbler( bitrate %u (%u), fold %u, mask 0x%02x )",
-                                options.bitrate, m_bitrate, m_fold, m_enable_mask );
+                LogMsg<2>( "+ BitBabbler( bitrate %u (%u), fold %u, mask 0x%02x [%02x] )",
+                    options.bitrate, m_bitrate, m_fold, options.enable_mask, m_enable_mask );
 
 
             unsigned    maxpacket = GetMaxPacketSize();
@@ -291,7 +291,7 @@ namespace BitB
             // maximum packet size and 64kB that will take less than 250ms to
             // transfer (which is then the maximum time we'll block waiting to
             // perform an orderly exit).
-            unsigned    chunksize = std::max( maxpacket,
+            size_t      chunksize = std::max( maxpacket,
                                     std::min( options.chunksize ? options.chunksize
                                                                 : 65536u,
                                               powof2_down(m_bitrate / 32
@@ -339,7 +339,7 @@ namespace BitB
             chunksize = SetChunkSize( chunksize );
             SetLatency( latency );
 
-            LogMsg<3>( "Chunk size %u, %u ms/per chunk (latency %u ms, max packet %u)",
+            LogMsg<3>( "Chunk size %zu, %zu ms/per chunk (latency %u ms, max packet %u)",
                         chunksize, chunksize * 8000 / m_bitrate, latency, maxpacket );
 
             if( claim_now )
@@ -577,7 +577,7 @@ namespace BitB
                     if( e == arg || *e != ':' || v > unsigned(-1) )
                         throw Error( _("Invalid --group-size option '%s'"), arg );
 
-                    groupid = v;
+                    groupid = Group::ID(v);
                     size    = StrToScaledUL( e + 1, 1024 );
                 }
 
@@ -771,6 +771,14 @@ namespace BitB
 
                 buf = new uint8_t[size];
 
+                // Bump the refcount until the thread is started, otherwise we
+                // may lose a race with this Source being released by the caller
+                // before the thread can take its handle from the raw pointer.
+                // Think of it as a virtual Handle passed with pthread_create.
+                Ref();
+
+                // We don't need to Unref() if this fails, because we'll throw
+                // and it will never have been constructed to be destroyed ...
                 int ret = pthread_create( &thread, GetDefaultThreadAttr(),
                                           Pool::source_thread, this );
                 if( ret )
@@ -1048,6 +1056,13 @@ namespace BitB
             }
             catch( const USBError &e )
             {
+                // Don't warn about enum values not being explicitly handled here,
+                // we don't want to have to chase every new error code added to
+                // libusb that we don't explicitly care about handling here.
+               #pragma GCC diagnostic push
+               #pragma GCC diagnostic ignored "-Wpragmas"
+               #pragma GCC diagnostic ignored "-Wswitch-enum"
+
                 switch( e.GetErrorCode() )
                 {
                     case LIBUSB_ERROR_PIPE:
@@ -1067,6 +1082,7 @@ namespace BitB
                     default:
                         throw;
                 }
+               #pragma GCC diagnostic pop
             }
 
         } //}}}
@@ -1076,6 +1092,9 @@ namespace BitB
 
             Source::Handle  s = static_cast<Source*>( p );
 
+            // Drop the 'virtual handle' from the ctor, we have a real one now.
+            s->Unref();
+
             try {
                 s->pool->do_source_thread( s );
             }
@@ -1360,7 +1379,7 @@ namespace BitB
 
                     if( ret )
                         LogErr<0>( ret,
-                                   "Pool::RemoveSource: failed to join thread for removed device %s\n",
+                                   "Pool::RemoveSource: failed to join thread for removed device %s",
                                    d->VerboseStr().c_str() );
                     else
                         Log<4>( "Pool::RemoveSource: joined thread for %s\n",
@@ -1445,7 +1464,7 @@ namespace BitB
                     if( w == 0 )
                         throw Error( _("Pool::WriteToFD( %d ) EOF"), fd );
 
-                    c -= w;
+                    c -= size_t(w);
                 }
 
                 if( len && (len -= n) == 0 )
@@ -1463,7 +1482,6 @@ namespace BitB
             pthread_t       p;
 
             int ret = pthread_create( &p, GetDefaultThreadAttr(), writefd_thread, w );
-
             if( ret )
             {
                 delete w;
@@ -1522,8 +1540,8 @@ namespace BitB
 
             const unsigned  N       = QA::FIPS::BUFFER_SIZE; // 20kbits for FIPS test.
             const unsigned  folds   = 2;
-            const int       timeout = m_opt.kernel_refill_time
-                                    ? m_opt.kernel_refill_time * 1000 : -1;
+            const int       timeout = int(m_opt.kernel_refill_time)
+                                    ? int(m_opt.kernel_refill_time * 1000) : -1;
 
             union {
                 uint8_t                 b[N + sizeof(struct rand_pool_info)];
@@ -1567,8 +1585,8 @@ namespace BitB
                 } while( ! source_ok || ! folded_ok );
 
 
-                rpi.entropy_count = n * 8;
-                rpi.buf_size = n;
+                rpi.entropy_count = int(n * 8);
+                rpi.buf_size      = int(n);
 
 
                #if EM_PLATFORM_LINUX
@@ -1615,7 +1633,6 @@ namespace BitB
             pthread_t       p;
 
             int ret = pthread_create( &p, GetDefaultThreadAttr(), feedkernel_thread, this );
-
             if( ret )
                 throw SystemError( ret, _("Pool::FeedKernelEntropyAsync: "
                                           "failed to create thread") );
@@ -1938,8 +1955,8 @@ namespace BitB
         { //{{{
 
             return new_device( vendorid, productid,
-                               StrToUL( busnum ? busnum : "", 10 ),
-                               StrToUL( devnum ? devnum : "", 10 ),
+                               StrToU( busnum ? busnum : "", 10 ),
+                               StrToU( devnum ? devnum : "", 10 ),
                                mfg     ? mfg : "",
                                product ? product : "",
                                serial  ? serial  : "",
