@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2010 - 2017,  Ron <ron@debian.org>
+//  Copyright 2010 - 2018,  Ron <ron@debian.org>
 
 #ifndef _BB_USBCONTEXT_H
 #define _BB_USBCONTEXT_H
@@ -119,7 +119,7 @@ namespace BitB
                     if( n == 0 )
                         vid = 0;
 
-                    else if( (vid = StrToUL( id.substr(0, n), 16 )) > 0xFFFF )
+                    else if( (vid = StrToU( id.substr(0, n), 16 )) > 0xFFFF )
                         throw 1;
 
                 } catch( const abi::__forced_unwind& ) {
@@ -132,7 +132,7 @@ namespace BitB
                     if( n + 1 == id.size() )
                         pid = 0;
 
-                    else if( (pid = StrToUL( id.substr(n + 1), 16 )) > 0xFFFF )
+                    else if( (pid = StrToU( id.substr(n + 1), 16 )) > 0xFFFF )
                         throw 1;
 
                 } catch( const abi::__forced_unwind& ) {
@@ -233,7 +233,7 @@ namespace BitB
                     if( n != std::string::npos )
                     {
                         try {
-                            if( (busnum = StrToUL( id.substr(0, n), 10 )) > 127 )
+                            if( (busnum = StrToU( id.substr(0, n), 10 )) > 127 )
                                 throw 1;
 
                         } catch( const abi::__forced_unwind& ) {
@@ -257,7 +257,7 @@ namespace BitB
                     if( n != std::string::npos )
                     {
                         try {
-                            if( (busnum = StrToUL( id.substr(0, n), 10 )) > 127 )
+                            if( (busnum = StrToU( id.substr(0, n), 10 )) > 127 )
                                 throw 1;
 
                         } catch( const abi::__forced_unwind& ) {
@@ -270,7 +270,7 @@ namespace BitB
                             throw Error( _("Device::ID: invalid device address '%s'"), id.c_str() );
 
                         try {
-                            devnum = StrToUL( id.substr(n + 1), 10 );
+                            devnum = StrToU( id.substr(n + 1), 10 );
 
                             if( devnum < 1 || devnum > 127 )
                                 throw 1;
@@ -289,7 +289,7 @@ namespace BitB
                     if( id.size() < 4 )
                     {
                         try {
-                            devnum = StrToUL( id, 10 );
+                            devnum = StrToU( id, 10 );
 
                             if( devnum < 1 || devnum > 127 )
                                 throw 1;
@@ -633,7 +633,7 @@ namespace BitB
                     {
                         libusb_config_descriptor    *c;
 
-                        int ret = libusb_get_config_descriptor( dev, i, &c );
+                        int ret = libusb_get_config_descriptor( dev, uint8_t(i), &c );
                         if( ret )
                             throw USBError( ret, _("USBContext::Device::Config: "
                                                    "failed to get configuration %zu descriptor"), i );
@@ -889,7 +889,7 @@ namespace BitB
                     if( __builtin_expect( config < 1 || config > 255, 0 ) )
                         throw Error( _("Device( %s ): invalid current config (1 < %d < 256)"),
                                                         m_device->IDStr().c_str(), config );
-                    return config;
+                    return uint8_t(config);
 
                 } //}}}
 
@@ -1000,7 +1000,7 @@ namespace BitB
                         return;
                     }
 
-                    int ret = libusb_clear_halt( m_handle, endpoint_address );
+                    int ret = libusb_clear_halt( m_handle, uint8_t(endpoint_address) );
                     if( ret )
                         throw USBError( ret, _("Device( %s ): ClearHalt failed for endpoint %02x"),
                                                     m_device->IDStr().c_str(), endpoint_address );
@@ -1333,7 +1333,7 @@ namespace BitB
             const std::string &GetDevicePort() const        { return m_devport; }
             const std::string &GetDevpath() const           { return m_devpath; }
 
-            unsigned GetNumConfigurations() const           { return m_configs.size(); }
+            unsigned GetNumConfigurations() const           { return unsigned(m_configs.size()); }
             const Config::Vector &GetConfigurations() const { return m_configs; }
 
             const Config &GetConfiguration( size_t n ) const
@@ -1484,10 +1484,10 @@ namespace BitB
             ScopedCancelState   cancelstate;
             Device::Handle      h;
             libusb_device     **devs;
-            int                 ret = libusb_get_device_list( m_usb, &devs );
+            ssize_t             ret = libusb_get_device_list( m_usb, &devs );
 
             if( ret < 0 )
-                throw USBError( ret, _("USBContext: failed to enumerate devices") );
+                throw USBError( int(ret), _("USBContext: failed to enumerate devices") );
 
             for( libusb_device **dev = devs; *dev; ++dev )
             {
@@ -1680,10 +1680,10 @@ namespace BitB
             ScopedCancelState   cancelstate;
 
             libusb_device     **devs;
-            int                 ret = libusb_get_device_list( m_usb, &devs );
+            ssize_t             ret = libusb_get_device_list( m_usb, &devs );
 
             if( ret < 0 )
-                throw USBError( ret, _("USBContext: failed to enumerate devices") );
+                throw USBError( int(ret), _("USBContext: failed to enumerate devices") );
 
             if( ! append )
                 m_devices.clear();
@@ -1693,10 +1693,10 @@ namespace BitB
                 libusb_device               *d = *dev;
                 libusb_device_descriptor    desc;
 
-                ret = libusb_get_device_descriptor(d, &desc);
-                if( ret < 0 )
+                int r = libusb_get_device_descriptor(d, &desc);
+                if( r < 0 )
                 {
-                    LogUSBError<1>( ret, _("USBContext::EnumerateDevices: failed to get descriptor") );
+                    LogUSBError<1>( r, _("USBContext::EnumerateDevices: failed to get descriptor") );
                     continue;
                 }
 
@@ -1728,7 +1728,7 @@ namespace BitB
         { //{{{
 
             ScopedMutex     lock( &m_device_mutex );
-            return m_devices.size();
+            return unsigned(m_devices.size());
 
         } //}}}
 
