@@ -1,5 +1,5 @@
 //  This file is distributed as part of the bit-babbler package.
-//  Copyright 2014 - 2017,  Ron <ron@debian.org>
+//  Copyright 2014 - 2018,  Ron <ron@debian.org>
 
 #include "private_setup.h"
 
@@ -8,22 +8,18 @@
 
 #include <bit-babbler/impl/log.h>
 
+#include <unistd.h>
 #include <getopt.h>
 
 
-#if EM_PLATFORM_POSIX
- #define DEFAULT_CONTROL_SOCK   "/var/run/bit-babbler/seedd.socket"
-#else
- #define DEFAULT_CONTROL_SOCK   "tcp:localhost:56789"
-#endif
-
-
 using BitB::Json;
 using BitB::ClientSock;
 using BitB::QA::Ent8;
 using BitB::QA::Ent16;
-using BitB::StrToUL;
+using BitB::StrToU;
+using BitB::StrToScaledU;
 using BitB::StrToScaledUL;
+using BitB::Error;
 using BitB::Log;
 using BitB::stringprintf;
 using std::string;
@@ -48,6 +44,7 @@ static void usage()
     printf("  -S, --stats               Report general QA statistics\n");
     printf("  -c, --control-socket=path The service socket to query\n");
     printf("  -V, --log-verbosity=n     Change the logging verbosity\n");
+    printf("      --waitfor=dev:n:r:max Wait for a device to pass some number of bytes\n");
     printf("  -v, --verbose             Enable verbose output\n");
     printf("  -?, --help                Show this help message\n");
     printf("      --version             Print the program version\n");
@@ -56,20 +53,88 @@ static void usage()
     printf("\n");
 }
 
+
+struct WaitFor
+{ //{{{
+
+    typedef std::list< WaitFor >    List;
+
+    std::string     deviceid;
+    size_t          bytes;
+    size_t          retry_ms;
+    size_t          timeout_ms;
+
+    WaitFor( const std::string &arg )
+        : retry_ms( 1000 )
+        , timeout_ms( 0 )
+    { //{{{
+
+        // Parse the options from a string of the form:
+        // device:bytes:retry_ms:timeout_ms
+        // Where device and bytes are mandatory.
+
+        size_t  n = arg.find( ':' );
+
+        if( n == string::npos )
+            throw Error( _("No byte count given in --waitfor=%s"), arg.c_str() );
+
+        deviceid = arg.substr( 0, n );
+        ++n;
+
+        try {
+            size_t  n2 = arg.find( ':', n );
+
+            if( n2 == string::npos )
+            {
+                bytes = StrToScaledUL( arg.substr(n), 1024 );
+                return;
+            }
+
+            bytes   = StrToScaledUL( arg.substr(n, n2 - n), 1024 );
+            n       = n2 + 1;
+            n2      = arg.find( ':', n );
+
+            if( n2 == string::npos )
+            {
+                retry_ms = StrToScaledUL( arg.substr(n) );
+                goto done;
+            }
+
+            retry_ms    = StrToScaledUL( arg.substr(n, n2 - n) );
+            n           = n2 + 1;
+            timeout_ms  = StrToScaledUL( arg.substr(n) );
+
+        done:
+            if( retry_ms < 1 )
+                throw Error( _("Retry time must be >= 1ms in --waitfor=%s"),
+                                                                arg.c_str() );
+        }
+        catch( const std::exception &e )
+        {
+            throw Error( _("Invalid --waitfor argument '%s': %s"),
+                                            arg.c_str(), e.what() );
+        }
+
+    } //}}}
+
+}; //}}}
+
+
 int main( int argc, char *argv[] )
 {
   try {
 
-    unsigned    opt_scan        = 0;
-    unsigned    opt_bin_count   = 0;
-    unsigned    opt_bin_freq    = 0;
-    unsigned    opt_bit_runs    = 0;
-    unsigned    opt_stats       = 0;
-    unsigned    opt_first       = 65536;
-    unsigned    opt_last        = 65536;
-    unsigned    opt_log_level   = unsigned(-1);
-    string      opt_deviceid;
-    string      opt_controlsock = DEFAULT_CONTROL_SOCK;
+    unsigned        opt_scan        = 0;
+    unsigned        opt_bin_count   = 0;
+    unsigned        opt_bin_freq    = 0;
+    unsigned        opt_bit_runs    = 0;
+    unsigned        opt_stats       = 0;
+    unsigned        opt_first       = 65536;
+    unsigned        opt_last        = 65536;
+    unsigned        opt_log_level   = unsigned(-1);
+    string          opt_deviceid;
+    string          opt_controlsock = SEEDD_CONTROL_SOCKET;
+    WaitFor::List   opt_wait;
 
     enum
     {
@@ -77,6 +142,7 @@ int main( int argc, char *argv[] )
         BINCOUNT16_OPT,
         FIRST_OPT,
         LAST_OPT,
+        WAITFOR_OPT,
         VERSION_OPT
     };
 
@@ -94,6 +160,7 @@ int main( int argc, char *argv[] )
         { "stats",          no_argument,        NULL,      'S' },
         { "control-socket", required_argument,  NULL,      'c' },
         { "log-verbosity",  required_argument,  NULL,      'V' },
+        { "waitfor",        required_argument,  NULL,      WAITFOR_OPT },
         { "verbose",        no_argument,        NULL,      'v' },
         { "help",           no_argument,        NULL,      '?' },
         { "version",        no_argument,        NULL,      VERSION_OPT },
@@ -139,14 +206,14 @@ int main( int argc, char *argv[] )
                 break;
 
             case FIRST_OPT:
-                opt_first = StrToScaledUL( optarg );
+                opt_first = StrToScaledU( optarg );
                 if( opt_last == 65536 )
                     opt_last = 0;
 
                 break;
 
             case LAST_OPT:
-                opt_last = StrToScaledUL( optarg );
+                opt_last = StrToScaledU( optarg );
                 if( opt_first == 65536 )
                     opt_first = 0;
                 break;
@@ -164,7 +231,11 @@ int main( int argc, char *argv[] )
                 break;
 
             case 'V':
-                opt_log_level = StrToUL( optarg, 10 );
+                opt_log_level = StrToU( optarg, 10 );
+                break;
+
+            case WAITFOR_OPT:
+                opt_wait.push_back( WaitFor(optarg) );
                 break;
 
             case 'v':
@@ -246,6 +317,66 @@ int main( int argc, char *argv[] )
     } //}}}
 
 
+    while( ! opt_wait.empty() )
+    { //{{{
+
+        const WaitFor   &w = opt_wait.front();
+        size_t          elapsed;
+
+        if( w.timeout_ms )
+            Log<1>( _("Waiting up to %zu ms for %zu good bytes from %s\n"),
+                                w.timeout_ms, w.bytes, w.deviceid.c_str() );
+        else
+            Log<1>( _("Waiting for %zu good bytes from %s\n"),
+                                              w.bytes, w.deviceid.c_str() );
+
+        for( elapsed = 0; w.timeout_ms == 0 || elapsed < w.timeout_ms;
+                                                        elapsed += w.retry_ms )
+        {
+            client.SendRequest( "[\"ReportStats\",1,\"" + w.deviceid + "\"]" );
+
+            Json::Handle    json = client.Read();
+
+            Log<4>("read reply: %s\n", json->JSONStr().c_str() );
+
+            if( json[0]->String() == "ReportStats" )
+            {
+                Json::Data::Handle  stats = json[2]->Get( w.deviceid );
+
+                if( ! stats )
+                    throw Error( _("No statistics available for device '%s'"),
+                                                         w.deviceid.c_str() );
+
+                unsigned long long  passed = stats["QA"]["BytesPassed"]->
+                                                    As<unsigned long long>();
+                if( passed >= w.bytes )
+                {
+                    Log<1>( _("Have %llu good bytes from %s in %zums\n"),
+                                    passed, w.deviceid.c_str(), elapsed );
+                    goto done;
+                }
+
+                Log<3>( _("Have %llu good bytes from %s in %zums (waiting for %zu)\n"),
+                                        passed, w.deviceid.c_str(), elapsed, w.bytes );
+            } else {
+
+                // Possibly we should throw here too, but since this should
+                // never happen, assume it's a glitch and just try again.
+                Log<0>( "Unrecognised reply to ReportStats request\n" );
+            }
+
+            usleep( useconds_t( w.retry_ms * 1000 ) );
+        }
+
+        throw Error( _("Timeout after %zums waiting for %zu bytes from %s\n"),
+                                    elapsed, w.bytes, w.deviceid.c_str() );
+
+    done:
+        opt_wait.pop_front();
+
+    } //}}}
+
+
     if( opt_bin_freq )
     { //{{{
 
@@ -392,8 +523,10 @@ int main( int argc, char *argv[] )
             for( Json::MemberList::iterator si = sources.begin(),
                                             se = sources.end(); si != se; ++si )
             {
-                unsigned long long  analysed = stats[*si]["QA"]["BytesAnalysed"];
-                unsigned long long  passed   = stats[*si]["QA"]["BytesPassed"];
+                unsigned long long  analysed = stats[*si]["QA"]["BytesAnalysed"]->
+                                                            As<unsigned long long>();
+                unsigned long long  passed   = stats[*si]["QA"]["BytesPassed"]->
+                                                            As<unsigned long long>();
                 BitB::QA::FIPS      fips( stats[*si]["FIPS"] );
 
                 printf( "\nsource: %s\n", si->c_str() );
